package NTLSystem.support;

import java.beans.*;
import java.util.*;

/**
 * An editor for MultivaluedAttributes.  This, along with
 * MultivaluedAtribute, shows how to create an editor which will work
 * with the PropertyCollectionSheet.
 * <P>
 * $Id: MultivaluedAttributeEditor.java,v 1.3 1998/06/19 01:36:53 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author    Josh Sled [jsled@icsi.berkeley.edu]
 * @see       MultivaluedAttribute
 * @see       PropertyCollectionSheet
 * @version   $Revision: 1.3 $
 **/
public class MultivaluedAttributeEditor
extends PropertyEditorSupport
{

  /**
   * Get the value of the currently-being-edited object.
   *
   * @return    A string representing the value of the editing
   *            target. 
   **/
  public String getAsText()
  {
    // Get the text of the current value.
    return ma.getValue();
  }

  /**
   * Sets the value of the currently-being-edited object.
   *
   * @param    text The value to which to set the editing target.
   **/
  public void setAsText(String text)
  {
    // Set the text of the current value.
    try {
      ma.setValue(text);
    } catch (Exception e) {
      throw new IllegalArgumentException(text);
    } // end try-catch
  }

  /**
   * Gets the tags for which this value can be set.  Returning
   * non-null for this method [of the PropertyEditor interface] means
   * that a choice [list] will be used for editing.
   *
   * @return    A String[] of the possible values for the target
   *            object. 
   **/
  public String[] getTags() {
    Enumeration e = ma.getPossibleValues();
    String toRet[] = new String[ma.getValueCount()];
    // use the enumeration to generate the tags.
    for (int i=0; e.hasMoreElements(); i++) {
      toRet[i] = (String)e.nextElement();
    } // end for over i, enumeration

    return toRet;
  }


  /**
   * Sets the object which is being edited by this editor.
   *
   * @param    o The target object to be edited.
   **/
  public void setValue(Object o)
  {
    ma = (MultivaluedAttribute)o;
  }

  /**
   * Gets the value of this object.
   * 
   * @return    The value of the object.
   **/
  public Object getValue()
  {
    return (Object)ma;
  }

  /** A holder for the currently-being-edited [target] object. **/
  MultivaluedAttribute ma;
  
}
