package NTLSystem.support;

import java.beans.*;
import java.lang.reflect.*;

/**
 *
 * An interface which objects can implement in order to be edited by
 * the PropertyCollectionSheet.
 * <P>
 * A PropertyCollection is an object which has a collection of
 * properties which can be edited.  These properties either have a
 * specific getter or setter method which is used to get or set their
 * value, respectively, or use a single getter/setter method, which is
 * indexed by a value specific to the property.  The
 * EditablePropertyDescriptors returned by objects implementing this
 * interface describe the behavior for each property in the
 * collection. 
 *
 * <P>
 *
 * For properties indexed through a single getter/setter method, the
 * methods are indexed by some type of object, which is the first 
 * argument to both the getter and setter method.  These indices can
 * be retreived for each Property through the
 * EditablePropertyDescriptor object for that property.
 *
 * <P>
 * $Id: PropertyCollection.java,v 1.3 1998/06/15 22:25:32 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @see EditablePropertyDescriptor
 * @see PropertyCollectionSheet
 * @version $Revision: 1.3 $
 *
 **/
public interface PropertyCollection {

  /**
   * A String which is used to name or label the PropertyCollection
   * being edited, described, etc.
   *
   * @return    A String labeling the PropertyCollection.
   **/ 
  public String getLabel();

  /**
   * The editable properties of this PropertyCollection.  This is an
   * array of EditablePropertyDescriptors which describe the properties.
   *
   * @return    An array of EditablePropertyDescriptor objects.
   *           
   **/
  public EditablePropertyDescriptor[] getEditableProperties();

}
