package NTLSystem.verblearn;

import java.io.*;
import java.util.*;
import NTLSystem;
import NTLSystem.support.*;

/**
 * A Sense of a Word is a collection of Features which describe that
 * Sense.  Therefore, it is an Fstruct.
 * <P>
 * From "Modeling Embodied Lexical Development":
 * <BLOCKQUOTE>
 * "Each Sense of a Verb is represented in the model by an Fstruct
 * whose values for each Feature are probability distributions.
 * <BR>[...]<BR>
 * With this scheme it is necessary to allow multiple senses for each
 * verb [Word], since capturing the meaning in a single Sense would in
 * some cases require overly broad distrubtions which obscure details
 * needed to carry out the corresponding action."
 * </BLOCKQUOTE>
 * <P>
 * <EM>This class needn't be public... it is only to get Javadoc 1.1
 * to generate docs for it.</EM>
 * <P>
 * $Id: Sense.java,v 1.2 1998/07/31 21:11:29 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author    David Bailey [dbailey@icsi.berkeley.edu]
 * @author    Nancy Chang [nchang@icsi.berkeley.edu]
 * @author    Josh Sled [jsled@icsi.berkeley.edu]
 * @see       Word
 * @see       Lexicon
 * @version   $Revision: 1.2 $
 **/
public class Sense
    extends Fstruct
    implements Serializable
{

    // {{{ ATTRIBUTES

    /** The Word which contains this Sense. **/
    private Word container;

    /** Number of examples covered by this Sense. **/
    private int numEx;

    // }}} end attributes

    // {{{ CONSTRUCTORS
    /**
     * Turn an instance into an initial sense
     *
     * @param     name Name of sense.
     * @param     f Fstruct on which sense is based. (Based on a Scenario)
     * @param     w The Word which contains this Sense.
     **/
    public Sense(String name, Fstruct f, Word w) {
	// FIXME... copy the Features and Properties.
	super(f);
	this.name = name;
	container = w;
	numEx = 1;
    }

    /**
     * Merge two Senses together into a single Sense, using the given
     * virtuals.
     *
     * @param     s1 The first Sense to merge.
     * @param     s2 The second Sense to merge.
     * @param     newSenseName The name of the merged Sense
     * @param     w The Word which contains this new Sense.
     **/
    public Sense(Sense s1,
		 Sense s2,
		 String newSenseName,
		 Word w)
    {
	// FIXME: Shouldn't this be: super( {s1, s2} ); and do
	// something with the virtuals?
	super(s1, s2);
	this.name = newSenseName;
	container = w;
	numEx = s1.numExamples() + s2.numExamples();
    }

    // }}} Constructors

    // {{{ Methods

    /**
     * Access method for number of examples this Sense covers.
     *
     * @return    The number of examples this Sense covers.
     **/
    public int numExamples() { return numEx; }

    // }}} Methods
}
