package NTLSystem.verblearn;

import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import NTLSystem.features.Fstruct;
import NTLSystem.support.*;


/**
 * The GUI functionality for a recognition test.
 *
 * $Id: RecognizeGUI.java,v 1.4 1998/10/16 01:59:38 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author    Joshua Sled [jsled@icsi.berkeley.edu]
 * @see       Lexicon
 * @see       ScenarioSet
 * @see       VerbLearn
 * @version   $Revision: 1.4 $
 **/
public class RecognizeGUI
    extends JInternalFrame
{
    
    /** The instance of VerbLearn we are associated with. **/
    private VerbLearn vl;
    
    /** The ScenarioSet of interest. **/
    private ScenarioSet sset;

    /** A combo box for selecting the lexicon of interest. **/
    private JComboBox lexicons;

    /** A ComboBox for selecting the Params to use. **/
    private JComboBox params;

    /** A Checkbox for selecting recognition with or without
	crunchy extra detail. **/
    private JCheckBox detail;

    /** RadioButtons for selecting the testing set. **/
    private JRadioButton allTestSet;
    private JRadioButton selectedTestSet;

    /** A ComboBox for selecting the dataset. **/
    private JComboBox dsets;
    /** A Vector of the datasets in the ScenarioSet. **/
    private Vector dsetNames;
    
    /** A List for selecting a subset of the test set. **/
    private JList testSetList;
    private JScrollPane testSetScroller;
    
    /** Where the recognition output will go. **/
    private JTextArea output;

    /** The button which performs recognition. **/
    private JButton recButton;

    /** The JButton to save the log. **/
    private JButton logSaveButton;
    /** The JButton to clear the log. **/
    private JButton logClearButton;

    /**
     * Constructor.
     *
     * @param     vl The instance of VerbLearn which we are contained
     *            within.
     * @param     initSset The ScenarioSet which is currently being
     *            edited. 
     **/
    public RecognizeGUI(VerbLearn vl, ScenarioSet initSset)
    {
	super("Recognition", true, true, true, true);

	this.vl = vl;
	this.sset = initSset;
	
	JComponent jc = (JComponent)getContentPane();

	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();

	jc.setLayout( gbl );

	// The top-left panel with the lexicon selection, lexicon
	// language label and recognition options.

	JPanel jp = new JPanel( new GridBagLayout() );
	
	c.gridwidth = GridBagConstraints.RELATIVE;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	JLabel jl = new JLabel( "Lexicon:" );
	((GridBagLayout)jp.getLayout()).setConstraints( jl, c );
	jp.add( jl );
	
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.HORIZONTAL;
	ComboBoxModel cbm = 
	    new VectorComboBoxModel(sset.getLexicons()) {
		// Adapt the VectorComboBoxModel to return the right
		// thing.
		public Object getElementAt(int i) {
		    return ((Lexicon)super.getElementAt(i)).getName();
		}
	    };
	lexicons = new JComboBox( cbm );
	lexicons.setSelectedIndex( 0 );
	((GridBagLayout)jp.getLayout()).setConstraints( lexicons, c );
	jp.add( lexicons );

	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	c.gridwidth = GridBagConstraints.RELATIVE;
	jl = new JLabel( "Parameters:" );
	((GridBagLayout)jp.getLayout()).setConstraints( jl, c );
	jp.add( jl );
	
	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridwidth = GridBagConstraints.REMAINDER;
	cbm = new ComboBoxModelAdaptor( sset.params );
	params = new JComboBox( cbm );
	params.addItemListener( new ItemListener() {
	    public void itemStateChanged(ItemEvent ie) {
		// Set the currentParams key in the ScenarioSet.
		sset.currentParamsKey = (String)params.getSelectedItem();
	    }
	} );
	((GridBagLayout)jp.getLayout()).setConstraints( params, c );
	jp.add( params );


	// Add sub-panel [with cool titled border] for recognize
	// options. 
	JPanel jp2 = new JPanel( new GridBagLayout() );
	jp2.setBorder( BorderFactory.
		       createTitledBorder( "Recognize" ) );

	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.HORIZONTAL;
	detail = new JCheckBox( "Detail", false );
	((GridBagLayout)jp2.getLayout()).setConstraints( detail, c );
	jp2.add( detail );

	ButtonGroup bg = new ButtonGroup();

	allTestSet =
	    new JRadioButton( "All test set scenarios", true );
	bg.add( allTestSet );
	((GridBagLayout)jp2.getLayout()).setConstraints( allTestSet,
							 c );
	jp2.add( allTestSet );

	selectedTestSet =
	    new JRadioButton( "Selected test set scenarios", false );
	bg.add( selectedTestSet );
	((GridBagLayout)jp2.getLayout()).
	    setConstraints( selectedTestSet, c );
	jp2.add( selectedTestSet );

	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.CENTER;
	((GridBagLayout)jp.getLayout()).setConstraints( jp2, c );
	jp.add( jp2 );
	
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.NONE;
	recButton = new JButton( "Recognize" );
	recButton.addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// Start recognition on a background thread.
		Thread t = new Thread() {
		    public void run() {
			recButton.setEnabled( false );
			recognize();
			recButton.setEnabled( true );
			JOptionPane.
			showMessageDialog(RecognizeGUI.this,
					  "Recognition done.",
					  "Recognition done.",
					  JOptionPane.INFORMATION_MESSAGE);
		    }
		}; // end Thread
		t.start();
	    }
	} );
	((GridBagLayout)jp.getLayout()).setConstraints( recButton,
							c );
	jp.add( recButton );

	jp2 = new JPanel();
	logSaveButton = new JButton( "Save Log" );
	logSaveButton.addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// Save the log to a specified file.
		// Get the file name from the user.
		JFileChooser jfc = new JFileChooser();
		jfc.setDialogTitle( "Save training log as..." );
		int status = jfc.showSaveDialog( RecognizeGUI.this.vl );
		if ( status != JFileChooser.CANCEL_OPTION ) {
		    File toSaveTo = jfc.getSelectedFile();
		    boolean cont = true;
		    if ( toSaveTo.exists() ) {
			cont = (JOptionPane.showConfirmDialog( RecognizeGUI.this.vl,
							       "Overwrite file " +
							       toSaveTo.getPath() +
							       "?",
							       "Overwrite file",
							       JOptionPane.YES_NO_OPTION)
				== JOptionPane.YES_OPTION);
		    } // end if
		    if (cont) {
			try {
			    FileWriter fw =
				new FileWriter( toSaveTo );
			    Utility.SaveTextArea( output, fw );
			    fw.close();
			    JOptionPane.showMessageDialog( RecognizeGUI.this.vl,
							   "Done saving log.",
							   "Done saving log.",
							   JOptionPane.INFORMATION_MESSAGE );
			} catch (IOException ioe) {
			    JOptionPane.showMessageDialog( RecognizeGUI.this.vl,
							   "Couldn't save file " +
							   toSaveTo.getPath() +
							   " because " +
							   ioe.getMessage(),
							   "File save error",
							   JOptionPane.ERROR_MESSAGE );
			} // end-catch
		    } // end cont
		} // end-if
		// Output the contents of the log to the file.
	    }
	} );
	jp2.add( logSaveButton );

	logClearButton = new JButton( "Clear Log" );
	logClearButton.addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// Clear the log...
		Document d = output.getDocument();
		try {
		    d.remove( 0, d.getLength() );
		} catch (BadLocationException ble) {
		    // We don't expect this because of the brain-dead
		    // arguments we're using, but it might happen.
		    throw new InternalError( ble.getMessage() );
		} // end try-catch
		output.validate();
	    }
	} );
	jp2.add( logClearButton );

	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.CENTER;
	((GridBagLayout)jp.getLayout()).setConstraints( jp2, c );
	jp.add( jp2 );

	c.gridwidth = GridBagConstraints.RELATIVE;
	c.fill = GridBagConstraints.BOTH;
	c.weightx = c.weighty = 1.0;
	c.anchor = GridBagConstraints.CENTER;
	gbl.setConstraints( jp, c );

	jc.add( jp );


	// Add the top-right panel with the dataset selection and the
	// dataset test-set list.
	jp = new JPanel( new GridBagLayout() );
       
	c.gridwidth = GridBagConstraints.RELATIVE;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	jl = new JLabel( "Dataset:" );
	((GridBagLayout)jp.getLayout()).setConstraints( jl, c );
	jp.add( jl );

	Enumeration e = sset.getDatasets();
	dsetNames = new Vector();
	while ( e.hasMoreElements() ) {
	    dsetNames.
		addElement( ((Dataset)e.nextElement()).getName() );
	} // end while over dataset enumeration
	cbm = new VectorComboBoxModel( dsetNames );
	dsets = new JComboBox( cbm );
	dsets.addItemListener( new ItemListener() {
	    public void itemStateChanged(ItemEvent ie) {
		// Change the test set list.
		String dsName = (String)dsets.getSelectedItem();
		Dataset ds = sset.getDataset( dsName );
		if (ds != null) {
		    Enumeration enum = ds.testEnum();
		    Vector v = new Vector();
		    while ( enum.hasMoreElements() ) {
			v.addElement( ((Scenario)enum.nextElement()).getName() );
		    } // end while over test set enumeration
		    testSetList.setListData( v );
		    // Update the GUI list
		    testSetScroller.validate();
		    testSetScroller.update( testSetScroller.getGraphics() );
		} // end if
	    }
	} );
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.WEST;
	((GridBagLayout)jp.getLayout()).setConstraints( dsets, c );
	jp.add( dsets );
	
	testSetList = new JList();
	testSetList.setSelectionMode( ListSelectionModel.
				      MULTIPLE_INTERVAL_SELECTION );
	testSetList.setVisibleRowCount( 10 );
	testSetList.setFixedCellWidth( 100 );
	testSetScroller =
	    new JScrollPane( testSetList,
			     JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			     JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.CENTER;
	((GridBagLayout)jp.getLayout()).
	    setConstraints( testSetScroller,
			    c );
	jp.add( testSetScroller );

	c.weightx = c.weighty = 1.0;
	gbl.setConstraints( jp, c );
	jc.add( jp );

	// Add the text area at the bottom for recognition output.

	c.fill = GridBagConstraints.BOTH;
	c.weightx = c.weighty = 1.0;
	output = new AppendScrollingTextArea( 25, 60 );
	output.setEditable( false );
	JScrollPane jsp =
	    new JScrollPane( output,
			     JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			     JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
	jsp.setBorder( BorderFactory.createTitledBorder( "Recognition log" ) );
	gbl.setConstraints( jsp, c );
	jc.add( jsp );

	// Pad the content pane a bit
	jc.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
	
	// Done... pack'n'show
	pack();
	setVisible( true );

    }

    protected void recognize() {
	if (params.getSelectedItem() == null) {
	    output.append("You must choose a parameter file before testing " +
			  "recognition.\n");
	    return;
	}
	if (dsets.getSelectedItem() == null) {
	    output.append("You haven't chosen a dataset!\n");
	    return;
	}

	Object[] scArray = null;
	// Vector<Scenario> to recognize against below.
	Vector toRecognize = new Vector();
	if ( selectedTestSet.isSelected() ) {
	    // Get the selected scenarios.
	    scArray = testSetList.getSelectedValues();
	    // Copy the elements from the array into the Vector
	    for (int i=0; i<scArray.length; i++) {
		toRecognize.
		    addElement( sset.getScenario( (String)scArray[i] ) );
	    } // end for over scArray
	} else {
	    // Get all the scenarios from the test set.
	    String dsetName = (String)dsets.getSelectedItem();
	    Dataset d = sset.getDataset( dsetName );
	    if ( d != null ) {
		Enumeration enum = d.testEnum();
		while ( enum.hasMoreElements() ) {
		    toRecognize.addElement( enum.nextElement() );
		} // end while overe test set numeration 
	    } // end if
	} // end if-else

	if ( toRecognize.size() == 0 ) {
	    output.append("You haven't chosen any scenarios to test!\n");
	    return;
	}

	output.append("Beginning recognition test...\n");
	int numCorrect = 0;
	int numTested = 0;

	Lexicon lex =
	    sset.getLexicon((String)lexicons.getSelectedItem());
	String lang = lex.getLanguage();
	boolean wantDetail = detail.isSelected();
	Fstruct scFstruct;
	
	for ( int i=0; i<toRecognize.size(); i++ ) {
	    Scenario sc = (Scenario)toRecognize.elementAt( i );
	    if (sc == null) {
		output.append( "Scenario is null!?\n" );
	    }
	    if (sc.isLabeled( lang )) {
		scFstruct = sc.getUnifiedFstruct();
		String label = sc.getLabel( lang ).toString();
		VerbComplex vc = lex.label( scFstruct );
		if (vc == null) {
		    output.append("  Returned VerbComplex is null.\n");
		} else {
		    String actual = vc.toString();
		    output.append("Scenario " +
				  sc.getName() +
				  ": desired=" + label +
				  ", output=" + actual);
		    numTested++;
		    if ( actual.equals( label ) ) {
			numCorrect++;
		    } else {
			output.append("    *ERROR*");
		    }
		    output.append("\n");
		}
		    // Print out full word probability vector only if
		    // detail is true.
		    if ( wantDetail ) {
			Vector v = lex.allSensePosteriors(scFstruct);
			for (int j=0; j<v.size(); j++) {
			    output.append("  ");
			    output.append( ((CandidateSense)v.
					    elementAt(j)).toString() );
			} // end for over CandidateSense vector from
			  // allSensePosteriors.
		    }
		} else {
		    output.append("WARNING: scenario " + sc.getName() +
				  " is unlabelled in language " + lang +
				  ".\n");
		}
	} // end for over scenarios to recognize.

	output.append("Correctly labelled " + numCorrect + " of " +
		      numTested + " test scenarios");
	if (numTested > 0) {
	    output.append(" (" +
			  ((int)((double)numCorrect / (double)numTested * 100)) + 
			  "%).\n");
	}
	output.append("Recognition test done.\n\n");
    }

}



