package NTLSystem.verblearn;

import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import com.sun.java.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import NTLSystem.*;
import NTLSystem.features.*;
import NTLSystem.support.VectorComboBoxModel;

/**
 * A GUI object which can be used to view and modify information
 * relating to a ScenarioSet. 
 * <P>
 * $Id: ScenarioSetViewer.java,v 1.2 1998/10/16 01:59:41 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author    Joshua Sled [jsled@icsi.berkeley.edu]
 * @see       ScenarioSet
 * @see       VerbLearn
 * @version   $Revision: 1.2 $
 **/
public class ScenarioSetViewer
    extends JInternalFrame
{
    // {{{ Attributes

    private ScenarioSet sset;

    private JTextField nameTF;
    private JTextArea descTA;
    private JList langs;
    private JTextField langToAddTF;
    private JButton addButton;
    private JPanel templateFstructs;
    private VectorComboBoxModel templateFstructVCBM;
    private VectorComboBoxModel langsVCBM;

    // }}} Attributes

    // {{{ Constructors
    
    public ScenarioSetViewer(ScenarioSet sset)
    {
	super("ScenarioSetViewer", true, true, true, true);

	this.sset = sset;

	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	
	JLabel jl;
	JComponent jc = (JComponent)getContentPane();
	JPanel jp, jp2;
	JButton jb;
	JScrollPane jsp;
	JComboBox jcb;
	GridBagLayout gblTmp, gblTmp2;
	
	// Set the layout
	jc.setLayout(gbl);
	
	gblTmp = new GridBagLayout();
	jp = new JPanel(gblTmp);

	c.fill = GridBagConstraints.NONE;
	c.gridwidth = GridBagConstraints.RELATIVE;
	jl = new JLabel("Name:");
	gblTmp.setConstraints(jl, c);
	jp.add(jl);

	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridwidth = GridBagConstraints.REMAINDER;
	nameTF = new JTextField( sset.getName(), 20);
	gblTmp.setConstraints(nameTF, c);
	jp.add(nameTF);

	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.CENTER;
	jl = new JLabel("Description:");
	gblTmp.setConstraints(jl, c);
	jp.add(jl);

	descTA = new JTextArea( sset.getDescription(), 10, 50 );
	// Make it line-wrap
	descTA.setLineWrap( true );
	// Make it word-wrap
	descTA.setWrapStyleWord( true );
	// Put it in a JScrollPane for scrolling.
	jsp = new JScrollPane( descTA,
			       ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
			       ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER    );
	gblTmp.setConstraints(jsp, c);
	jp.add( jsp );

	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(jp, c);
	jc.add(jp);

	gblTmp = new GridBagLayout();
	jp = new JPanel(gblTmp);
	jp.setBorder(BorderFactory.createTitledBorder("Languages"));

	c.fill = GridBagConstraints.BOTH;
	c.gridwidth = GridBagConstraints.REMAINDER;
	langsVCBM = new VectorComboBoxModel( sset.getLanguages() );
	langs = new JList( langsVCBM );
	// Put the list in a scroll pane for scrolling ability.
	jsp = new JScrollPane( langs );
	gblTmp.setConstraints(jsp, c);
	jp.add(jsp);

	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridwidth = GridBagConstraints.RELATIVE;
	langToAddTF = new JTextField( 20 );
	// <RET> on TextField should => "add"
	langToAddTF.addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// Pretend that we clicked the 'add' button...
		addButton.doClick();
	    }
	});
	gblTmp.setConstraints(langToAddTF, c);
	jp.add(langToAddTF);

	c.fill = GridBagConstraints.NONE;
	c.gridwidth = GridBagConstraints.REMAINDER;
	addButton = new JButton("Add");
	addButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		Vector langs = ScenarioSetViewer.this.sset.getLanguages();
		// Get the value from the TF ...
		String toAdd = langToAddTF.getText();
		// ... trim it ...
		toAdd = toAdd.trim();
		// ... see if it already exists ...
		if ( langs.contains( toAdd ) ) {
		    JOptionPane.showMessageDialog(ScenarioSetViewer.this,
						  "Language " + toAdd
						  + " already in use.",
						 "Language in use.",
						  JOptionPane.WARNING_MESSAGE);
		} else {
		    // If not, add it to the language vector...
		    ScenarioSetViewer.this.sset.addLanguage( toAdd );
		    // ... and remove it from the text field...
		    langToAddTF.setText("");
		    // ... and update the list.
		    langsVCBM.update();
		}
	    }
	});
	gblTmp.setConstraints(addButton, c);
	jp.add( addButton );

	c.fill = GridBagConstraints.NONE;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.CENTER;
	jb = new JButton("Remove Selected");
	jb.addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// FIXME
		// Get the selected languages.
		// Confirm, displaying effects...
		// 1. Remove selected languages
		// 2. Remove same- lexicons?
		// 3. Remove labels from ScenarioSets.

		// Update panel
		// Update combo box.
	    }
	});
	gblTmp.setConstraints(jb, c);
	jp.add(jb);

	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.CENTER;
	gbl.setConstraints(jp, c);
	jc.add(jp);

	gblTmp = new GridBagLayout();
	jp = new JPanel(gblTmp);
	jp.setBorder( BorderFactory.createTitledBorder("Template Fstructs") );

	gblTmp2 = new GridBagLayout();
	jp2 = new JPanel(gblTmp2);
	    
	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridwidth = GridBagConstraints.RELATIVE;
	c.gridheight = 2;
	jb = new JButton("Add new Fstruct");
	jb.addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// Add new empty Fstruct to ScenarioSet template
		//   vector
		// Get the name from the user.
		String newFstructName =
		    JOptionPane.showInputDialog("Please give this new Fstruct a name.");
		// Pass the correct FeatureDictionary to the Fstruct
		Fstruct toAdd = new Fstruct( newFstructName,
					     VerbLearn.featureDict );
		ScenarioSetViewer.this.sset.addTemplateFstruct( toAdd );
		// Update the combo box model
		templateFstructVCBM.update();
		// Add to the scrollableFstructs panel
		FstructViewer fsv = new FstructViewer( toAdd );
		fsv.setBorder( BorderFactory.createLineBorder( Color.black ) );
		((GridBagLayout)templateFstructs.getLayout()).
		    setConstraints( fsv, FstructViewer.panelConstraints );
		templateFstructs.add( fsv );
		// Update scrollable fstructs panel by validating the
		// internal frame. 
		ScenarioSetViewer.this.validate();
	    }
	});
	gblTmp2.setConstraints(jb, c);
	jp2.add(jb);
	    
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.gridheight = 1;
	
	templateFstructVCBM =
	    new VectorComboBoxModel(ScenarioSetViewer.this.sset.getTemplateFstructs() ) {
		public Object getElementAt(int i) {
		    return ((Fstruct)vec.elementAt(i)).getName();
		}
	    };
	jcb = new JComboBox( templateFstructVCBM );
	gblTmp2.setConstraints(jcb, c);
	jp2.add(jcb);

	c.gridwidth = GridBagConstraints.REMAINDER;
	jb = new JButton("Remove Fstruct");
	jb.addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// Remove named Fstruct from the scenario set.
		Fstruct f =
		    ScenarioSetViewer.this.sset.
		    getTemplateFstruct( (String)templateFstructVCBM.getSelectedItem() );

		ScenarioSetViewer.this.sset.
		    removeTemplateFstruct( f );

		// Do the same for the templateFstruct panel
		boolean done = false;
		Component[] cs = templateFstructs.getComponents();
		for (int i=0; i<cs.length && !done; i++) {
		    if ( ((FstructViewer)cs[i]).getTarget().
			 getName().equals( f.getName() ) ) {
			templateFstructs.remove( cs[i] );
			done = true;
		    }
		} // end for over i
		// Update the combobox
		templateFstructVCBM.update();
		// Update the GUI
		ScenarioSetViewer.this.validate();
	    }
	});
	gblTmp2.setConstraints(jb, c);
	jp2.add(jb);

	c.fill = GridBagConstraints.NONE;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.CENTER;
	gblTmp.setConstraints(jp2, c);
	jp.add(jp2);

	templateFstructs = new JPanel( new GridBagLayout() );

	c.fill = GridBagConstraints.BOTH;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.weightx = c.weighty = 1.0;
	// Initially fill Fstructs with values from
	// ScenarioSet's Vector of template Fstructs.
	Vector templates = sset.getTemplateFstructs();
	FstructViewer fsv;
	for (int i=0; i<templates.size(); i++) {
	    fsv = new FstructViewer( (Fstruct)templates.elementAt(i) );
	    fsv.setBorder( BorderFactory.createLineBorder( Color.black ) );
	    ((GridBagLayout)jp.getLayout()).
		setConstraints( fsv, FstructViewer.panelConstraints );
	    templateFstructs.add( fsv );
	} // end for over i
	jsp = new JScrollPane( templateFstructs,
			       ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED ,
			       ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS );
	Dimension size = new Dimension( 700, 420 );
	jsp.setPreferredSize( size );
	jsp.setMaximumSize( size );
	gbl.setConstraints(jsp, c);
	jp.add(jsp);

	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.NORTH;
	c.weightx = c.weighty = 1.0;
	gbl.setConstraints(jp, c);
	jc.add(jp);

	// Add an internal frame listener which saves values upon
	// window closing.
	addInternalFrameListener( (InternalFrameListener)(new SaveOnClose()) );

	pack();
	setVisible(true);
    }

    // }}} Constructors

    // {{{ Methods


    // }}} Methods

    // {{{ Inner classes

    // {{{ SaveOnClose

    /**
     * Inner class to save values upon the closing of this internal
     * frame. 
     **/
    protected class SaveOnClose
	extends InternalFrameAdapter
    {
	public SaveOnClose()
	{
	    super();
	}

	public void internalFrameClosing(InternalFrameEvent ife)
	{
	    // Save the values which haven't been saved yet...
	    // This includes the name text field and the description
	    // text area.  The langauges and template Fstructs are
	    // modified in-place.
	    if (nameTF.getText().equals("")) {
		JOptionPane.showMessageDialog(ScenarioSetViewer.this,
					      "Scenario Set must have a name.",
					      "Invalid Scenario Set name.",
					      JOptionPane.ERROR_MESSAGE);
	    }
	    sset.setName( nameTF.getText() );
	    sset.setDescription( descTA.getText() );
	}

    }

    // }}} SaveOnClose

    // }}} Inner classes

}
