package NTLSystem.verblearn;

import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import NTLSystem.support.*;


///////////////////////////////////////////////////////////////////////
// A panel containing a grid of buttons, one for each word in the
// lexicon, arranged as one column per slot, plus text fields across
// the top which reflect the most recently selected word in each slot.

class CommandEntryGrid extends Panel {

  public static final int MAX_PER_COLUMN = 6;

  private Label warn;
  private TextField[] command;
  private Hashtable buttonTable;  // maps Buttons to slotNums

  CommandEntryGrid() {
    setLayout(new GridBagLayout());
    rebuild();
  }

  public void rebuild() {
    // Get rid of any old components:
    if (warn != null) {
      remove(warn);
      warn = null;
    }
    else if (buttonTable != null) {
      Enumeration bEnum = buttonTable.keys();
      while (bEnum.hasMoreElements()) {
	remove((Button)bEnum.nextElement());
      }
      for (int i = 0; i < command.length; i++) {
	remove(command[i]);
      }
      buttonTable = null;
      command = null;
    }
    // Insert a new set of components:
    if (VerbLearn.context().lexicon() == null) {
      warn = new Label("(No lexicon exists yet)");
      Constrain.add(this, warn, 0, 0, 1, 1, 10, 10, 10, 10);
    }
    else {
      buttonTable = new Hashtable();
      command = new TextField[VerbLearn.context().numSlots()];
      int curCol = 0;
      for (int s = 0; s < VerbLearn.context().numSlots(); s++) {
	command[s] = new TextField(8);
	command[s].setEditable(false);
	Constrain.add(this, command[s], curCol, 0, 1, 1, 5, 5, 5, 5);
	Button b = new Button("      ");
	b.setFont(new Font("Helvetica", Font.PLAIN, 10));
	Constrain.add(this, b, curCol, 1, 1, 1, 3, 5, 3, 5);
	buttonTable.put(b, new Integer(s));
	String[] words = VerbLearn.context().lexicon().slot(s).wordList();
	int curRow = 1;
	for (int w = 0; w < words.length; w++) {
	  b = new Button(words[w]);
	  b.setFont(new Font("Helvetica", Font.PLAIN, 10));
	  Constrain.add(this, b, curCol, curRow + 1, 1, 1, 3, 5, 3, 5);
	  buttonTable.put(b, new Integer(s));
	  curRow++;
	  if (curRow == MAX_PER_COLUMN) {
	    curCol++;
	    curRow = 0;
	  }
	}
	curCol++;
      }
    }
    // validate();
  }

  public void clear() {
    if (warn == null) {
      for (int s = 0; s < VerbLearn.context().numSlots(); s++) {
	command[s].setText("");
      }
    }
  }

  public boolean action(Event e, Object arg) {
    int s = ((Integer)buttonTable.get(e.target)).intValue();
    if (s >= 0) {
      command[s].setText(((Button)e.target).getLabel().trim());
      return true;
    }
    return super.action(e, arg);
  }

  public String getCommand() {
    String str = "";
    for (int i = 0; i < command.length; i++) {
      if (i > 0) str += "-";
      str += command[i].getText();
    }
    return str;
  }
}
