package NTLSystem.verblearn

import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import NTLSystem.support.*;


/*  

    This file contains the class GenPanel.
    As the file was becoming rather bloated,
    the popupDialogs and related classes were 
    moved into a separate file, scenarioViewer.java
    Some of the code from David Bailey's Generator class
    has been incorporated into GenPanel and generatorThread.


    Oliver King
    king@math.berkeley.edu
    12/9/1997 , revised 5/20/1997

*/

public class GenPanel extends Panel implements Observer {
  
  /** standard restrictions filename */
  String standardRestrictions = "standardrestrictions";

  /** might want to add restrictions directory to VerbLearn,
      and change this */
  String directoryForRestrictions = VerbLearn.DATA_BASE_DIR + "scenario";
  ////////////////////////////////////////

  /** default values for TextFields: */

  /** default value for nameBox */
  String defaultNameStem = "sc";

  /** default value for numberBox */ 
  String defaultNumberOfScenarios = "50";

  /** default value for firstNumberBox */
  String defaultStartingIndex = "1";

  /** default value for pruneBox */
  String defaultPrunePercentage = "100";

  ///////////////////////////////////////
  
  /** top level frame */
  Frame parentFrame;
  
  /** The Params from which the permissible features and values are drawn */
  Params params;
 
  /** stores generated (or loaded) Scenarios */
  Vector scenarioVector;

  ScenarioSet scenSet;

  Vector selectedVector; // stores names of selected scenes
    
  /** stores restrictions, which are Fstructs */
  Vector exclusionVector; 
  
  /** number of Scenarios which have been rejected,
      i.e., violated one of the restrictions  */
  int rejectCount = 0;
  
  /** flag for aborting scenario-genaring process */
  boolean stopMakingScenarios = false;

  /** Currently selected feauture in featureList */
  String selectedFeature = "";
  
  /** index of the selected feature in featureList */
  int selectedIndex = -1;
  
  /** number of scenarios to be generated */
  int numberOfScenarios;

  /** starting index for Scenarios to be generated */
  int startingIndex;
  
  /** stem of the names for the Scenarios to be generated */
  String nameStem;
 
  /** keyed by feature names, value is another Hashtable (which is keyed by
      the values for that feature, and whose value is an Integer weight */
  Hashtable weightTable;

  /** Dialog for choosing which Params to use.
      Currently need to reside on param directory */
  FileDialog paramDialog;

  /** Dialog for saving current batch of Scenarios.
      Or for loading existing Scenarios  */
  FileDialog scenarioDialog;
 
  /** for cancelling an in-progress scenario-generation */ 
  scenarioPopupDialog scenarioGenDialog;

  /** for setting values of selected feature in featureList */
  PopupMenu valuePopup;

  // popup menu for selecting subset of scenarios set 
  PopupMenu selectionPopup;
  
  // options for selsectionPopup
  MenuItem selAll;
  MenuItem selNone;
  MenuItem selCurrent;
  MenuItem selRestricted;
  MenuItem selComplement;
  Menu selFraction;



  // nc added
  removeFeatureDialog removeFeatDialog;
  
  public TextField status; // status bar

  ////////////////////////////////////////////////////////////
  
  /** holds controlPanel and featurePanel */
  Panel topPanel;

  /** holds widgets for generating, loading and saving scenarios */
  Panel controlPanel;
  Panel paramPanel;
  
  /** for choosing which Params to use */
  Choice paramChoice;
  
  /** holds number of Scenarios to be generated */
  TextField numberBox;

  /** holds stem for the names of the Scenarios to be generated */
  TextField nameBox;
  
  /** holds starting index for Scenarios to be generated */
  TextField firstNumberBox;

  /** for saving current batch of Scenarios */
  Button saveButton;
 
  /** for generating new Scenarios */
  Button genButton; 

  /** for loading existing scenarios.
      currently only works for default, "scen" */
  Button loadButton;

  //  Button addFeatureButton; 
  Button changeFeatureButton; 
  
  ///////////////////////////////////////////////////////////
  
  /** holds widgets relating to restrictions on scenarios */
  Panel exclusionPanel;

  /** displays exclusions (which are stored in exlusionVector) */ 
  List exclusionList;
 
  /** brings up exlusionPopupDialog */ 
  Button addExclusionButton;
  
  /** removes selected restriction(s) in exclusionList */
  Button removeExclusionButton;

  /** loads restrictions from file*/
  Button loadOtherExclusionsButton;

  /** saves restrictions */ 
  Button saveExclusionsButton;

  ////////////////////////////////////////////////////////////
  
  /** holds widgets relating to setting values or their relative
      frequencies */
  Panel featurePanel;

  /** list of world and motor features in the Params being used */
  List featureList; 

  ////////////////////////////////////////////////////////////
  
  /** holds widgets for viewing and filtering current Scenarios */
  Panel scenarioPanel; 
    
  /** renames Scenarios to eliminate gaps caused by pruning */
  Button compactNamesButton;

  /** deletes (with some probabilty) the selected Scenarios in 
      scenarioList */
  Button pruneSelectedButton;
  
  /** deletes (with some probabilty) the unselected Scenarios 
      in scenarioList */
  Button pruneUnselectedButton;
  
  /** selects all Scenarios in scenarioList which violate a 
      restriction in exclusionList */
  Button selectRestrictedButton;
  
  /** deselects all the Scenarios in scenarioList */
  Button deselectButton;
  
  /** holds the prune factor -- the probabilty (in percent) that
      a selected (or unselected) Scenario will be deleted when pruning 
      the scenarioList */ 
  TextField pruneBox;
  
   
  ////////////////////////////////////////////////////////

  
  /** the dialog windows that get popped-up need a Frame in their 
      constructors, so we pass this panel its parent frame for
      later reference */
  
  public GenPanel(Frame frame){
    
     super();

     VerbLearn.context().addObserver(this);

     setSize(780,780);
     parentFrame = frame;
     setBackground( new Color(0xddddff));
     setLayout(new BorderLayout());

     // take care of top row
     topPanel = new Panel();
     topPanel.setLayout(new FlowLayout());
     topPanel.setSize(700,300);
     Panel middlePanel = new Panel();
     middlePanel.setLayout(new FlowLayout());
     middlePanel.setSize(700, 300);
     buildControlPanel();
     buildFeaturePanel();
     buildExclusionPanel();
     buildScenarioPanel();
     topPanel.add(controlPanel, "West");
     topPanel.add(exclusionPanel, "East");
     middlePanel.add(featurePanel, "West");
     middlePanel.add(scenarioPanel, "East");
     add(topPanel, "North");
     add(middlePanel, "Center");
     status = new
       TextField("Generate scenarios or add/remove features.", 90);
     status.setEditable(false);
     add(status, "South");
    
     exclusionVector = new Vector();
     scenarioVector = new Vector();
     selectedVector = new Vector();
     // hack -- can't find file so creates empty scenario set
     scenSet = new ScenarioSet("nonexistent");
     updateButtonStates();
     validate(); repaint();
     //setVisible(true);
     //     paramChoice.remove("defaultparam");
     //for (int i = 0; i < VerbLearn.context().paramList().length; i++) {
     //       paramChoice.addItem(VerbLearn.context().paramList()[i]);
     //}

     buildParamChoice();  // rebuild gui
     changeParams(); //loads context's params, but can be changed by user
     //showParameterFileDialog();
  }
  
  // called when context notifies observers
  public void update(Observable o, Object junk) {
    Context c = (Context)o;
    String preservedChoice = c.params().id(); ; // not there yet
    buildParamChoice();  // rebuild gui based on new context
    changeParams(); // based on whatever has been chosen = context
    paramChoice.select(preservedChoice); // might be unnecessary
    paramChoice.hide(); paramChoice.show();
    // this takes care of following lines
    //    params = VerbLearn.context().params();
    //      buildFeatureList();
    //      buildWeightTable();
    // add call...
    VerbLearn.context().scenarios().rearrangeFstructs();
    fstructViewer.reconfigure(params, true, false, false, false);
  }


   MenuItem deselCurrent, compactNames, deleteSelected,
            selFrac25, selFrac50, selFrac75, merge;

   public void buildSelectionPopup(){

      selectionPopup = new PopupMenu();
  
      selAll = new MenuItem("select all");
      selNone = new MenuItem("select none");
      selCurrent = new MenuItem("select current");
      deselCurrent = new MenuItem("deselect current");
      selRestricted = new MenuItem("select restricted");
      selComplement = new MenuItem("select complement");
      selFraction = new Menu("select fraction...");
      selFrac75 = new MenuItem("75% of selected");
      selFrac50 = new MenuItem("50% of selected");
      selFrac25 = new MenuItem("25% of selected");
      compactNames = new MenuItem("compact names");
      deleteSelected = new MenuItem("delete selected scenarios");
      // might want better name for this
      merge = new MenuItem("merge into label panel");
      
      selectionPopup.add(selAll);
      selectionPopup.add(selNone);
      selectionPopup.add(selCurrent);
      selectionPopup.add(deselCurrent);
      selectionPopup.add(selRestricted);
      selectionPopup.add(selComplement);
      selectionPopup.add(selFraction);
      selFraction.add(selFrac75);
      selFraction.add(selFrac50);
      selFraction.add(selFrac25);
      selectionPopup.addSeparator();
      selectionPopup.add(deleteSelected);
      selectionPopup.addSeparator();
      selectionPopup.add(compactNames);
      selectionPopup.addSeparator();
      selectionPopup.add(merge);

     
      add(selectionPopup);

  }
  
  // update (or initialize) choice gui -- add current paramlist
  // from current context (updated already
  public void buildParamChoice() {
    paramChoice.removeAll();
    for (int i = 0; i < VerbLearn.context().paramList().length; i++) {
      paramChoice.addItem(VerbLearn.context().paramList()[i]);
    }
    validate(); paramPanel.repaint(); controlPanel.repaint(); repaint();
    paramChoice.select(VerbLearn.context().params().id());
    paramChoice.hide(); paramChoice.show();
    // ****
  }

  /** returns top level frame */
  
  public Frame getParentFrame(){
     return (Frame) getParent();
  }
  
  /** makes and lays-out widgets in scenarioPanel */
  FstructViewer fstructViewer;

  Button fastForward;
  Button forward;
  Button rewind;
  Button fastRewind;
  Button deleteScenario;
  Button copyScenario;
  Button addNewScenario;
  Choice scenarioChoice;
  SelectoMeter selectoMeter;
  Button scenarioSelect;
  
  public void buildScenarioPanel(){
     scenarioPanel = new Panel();
     scenarioPanel.setSize(700, 300);
     scenarioPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 20, 10));
     compactNamesButton = new Button("compact names");
     pruneSelectedButton = new Button ("prune selected scenarios");
     pruneUnselectedButton = new Button ("prune unselected scenarios");
     selectRestrictedButton =  new Button ("select restricted scenarios");
     deselectButton = new Button("deselect all");
     Panel topPanel = new Panel();
     topPanel.setLayout(new BorderLayout(3,3));
     pruneBox = new TextField (defaultPrunePercentage);
     Panel buttonPanel = new Panel();
     buttonPanel.setLayout(new GridLayout(5, 1, 5, 5));
     buttonPanel.add(compactNamesButton);
     buttonPanel.add(deselectButton);
     buttonPanel.add(selectRestrictedButton);
     buttonPanel.add(pruneSelectedButton);
     buttonPanel.add(pruneUnselectedButton);
     Panel prunePanel = new Panel();
     prunePanel.add(new Label("pruning factor:"));
     prunePanel.add(pruneBox);
     prunePanel.add(new Label("%"));
     Panel leftPanel = new Panel();
     Panel rightPanel = new Panel();
     leftPanel.setLayout(new BorderLayout());
     rightPanel.setLayout(new BorderLayout());
     
     fstructViewer = new FstructViewer();
     rightPanel.add(fstructViewer, "North");

     scenarioChoice = new Choice();
     scenarioChoice.add("<name>");
     selectoMeter = new SelectoMeter();
     Panel meterPanel = new Panel();
     meterPanel.setLayout(new BorderLayout());
     meterPanel.add(scenarioChoice, "West");
     meterPanel.add(selectoMeter, "East"); 

     rightPanel.add(meterPanel, "Center");
     Panel buttonPanel2 = new Panel();
     buttonPanel2.add(scenarioSelect = new Button("options..."));
     buttonPanel2.add(fastRewind = new Button("<<"));
     buttonPanel2.add(rewind = new Button("<"));
     buttonPanel2.add(forward = new Button(">"));
     buttonPanel2.add(fastForward = new Button(">>"));
     buttonPanel2.add(addNewScenario = new Button("+"));
     buttonPanel2.add(copyScenario = new Button("&"));
     buttonPanel2.add(deleteScenario = new Button("-"));
     
     rightPanel.add(buttonPanel2, "South");
     scenarioPanel.add(rightPanel);
  }
  
  /** makes and lays-out widgets in featurePanel */
  
  public void buildFeaturePanel(){
     featurePanel = new Panel();
     featurePanel.setLayout(new BorderLayout());
     Label featureLabel = 
        new Label("Required values:", Label.CENTER);
     featureList = new List(10, false);
     featurePanel.add(featureLabel, "North");
     featurePanel.add(featureList, "Center");
     changeFeatureButton = new Button("add/remove features");
     featurePanel.add(changeFeatureButton, "South");
  }

  /** makes and lays-out widgets in exclusionPanel */
  
  public void buildExclusionPanel(){
     exclusionPanel = new Panel();
     exclusionPanel.setSize(400, 200);
     // exclusionPanel.setBackground(Color.green);
     exclusionPanel.setLayout(new BorderLayout()); 

     Label exclusionLabel =
         new Label("Restrictions:", 
                   Label.LEFT);
     exclusionList = new List(4, true);

     
     Panel buttonPanel = new Panel();
     buttonPanel.setLayout(new GridLayout(1, 4, 5, 5));
     addExclusionButton = new Button("new");
     removeExclusionButton = new Button("remove");
     loadOtherExclusionsButton = new Button("load");
     saveExclusionsButton = new Button("save");

     buttonPanel.add(addExclusionButton);
     buttonPanel.add(loadOtherExclusionsButton);
     buttonPanel.add(saveExclusionsButton);
     buttonPanel.add(removeExclusionButton);

     Panel topPanel = new Panel();
     topPanel.setLayout(new BorderLayout());
     topPanel.add(exclusionLabel, "West");
     topPanel.add(buttonPanel, "East");
     
     exclusionPanel.add(topPanel, "North");
     exclusionPanel.add(exclusionList, "Center");
  }
     
  /** makes and lays-out widgets in controlPanel */

  public void buildControlPanel(){
     controlPanel = new Panel();
     //controlPanel.setBackground(Color.green);
     controlPanel.setLayout(new GridBagLayout());

     //     controlPanel.setLayout(new BorderLayout(4,4));
     //     controlPanel.setSize(200,200);
     loadButton = new Button("load scenarios");
     saveButton = new Button("save scenarios");

     genButton = new Button("generate scenarios");

     //     addFeatureButton = new Button("add feature");
     //     changeFeatureButton = new Button("Add or remove features");

     Panel buttonPanel = new Panel();
     Panel buttonPanel2 = new Panel();

     buttonPanel.setLayout(new GridLayout(1, 2, 5, 5));
     buttonPanel.add(loadButton);
     buttonPanel.add(saveButton);

     //     buttonPanel2.setLayout(new GridLayout(1, 2, 5, 5));
     //     buttonPanel2.add(addFeatureButton);
     //     buttonPanel2.add(removeFeatureButton);

     numberBox = new TextField(defaultNumberOfScenarios, 5);
     firstNumberBox = new TextField(defaultStartingIndex, 5);
     nameBox = new TextField(defaultNameStem, 5);
     Panel pane = new Panel();
     pane.setLayout(new GridLayout(3, 2, 4, 4));
     pane.add(new Label("how many ", Label.RIGHT)); 
     pane.add(numberBox);
     pane.add(new Label("first index ", Label.RIGHT)); 
     pane.add(firstNumberBox);
     pane.add(new Label("name stem ", Label.RIGHT)); 
     pane.add(nameBox);

     paramChoice = new Choice();
     paramPanel = new Panel();
     paramPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 0,0));
     paramPanel.add(new Label("Parameter file:"));
     paramPanel.add(paramChoice);
     buildParamChoice(); // adds appropriate param files


     //top left bottom right
     Constrain.add(controlPanel,buttonPanel, 0,0,1,1,5,5,10,5); 
     Constrain.add(controlPanel,genButton, 0,1,1,1,5,5,5,5); 
     Constrain.add(controlPanel,pane, 0,2,1,1,5,5,10,5); 
     Constrain.add(controlPanel,paramPanel, 0,3,1,1,5,5,5,5); 
     //changeFeatureButton = new Button("Add or remove features");
     //Constrain.add(controlPanel,changeFeatureButton, 0,4,1,1,5,5,5,5); 
   }


  /** brings up Dialog for saving current batch of Scenarios */
  
  public void showScenarioSaveDialog(){
    // DRB: Just hard-code the filename as "scenario/specs" for now.
//      scenarioDialog = 
//          new FileDialog(parentFrame, "save scenarios", FileDialog.SAVE);
//      scenarioDialog.setDirectory(VerbLearn.DATA_BASE_DIR + "scenario");
//      scenarioDialog.setBackground(Color.white);
//      scenarioDialog.show();
//      String scenarioFile = scenarioDialog.getFile();
//      String scenarioDirectory = scenarioDialog.getDirectory();
    String scenarioFile = "specs";
    String scenarioDirectory = VerbLearn.DATA_BASE_DIR + "scenario";
     if (scenarioFile != null) {
         try {
	   writeScenarioVector(new File(scenarioDirectory, scenarioFile)); 
	   status.setText("Saved scenarios.");
	 }
         catch (IOException ioe){
            status.setText("Error in saving Scenarios");
         }
     }
     VerbLearn.context().scenarios().incorporate(scenarioVector);
  }

  /** brings up Dialog for saving current set of restrictions */ 
 
  public void showExclusionSaveDialog(){
     scenarioDialog = 
         new FileDialog(parentFrame, "save restrictions", FileDialog.SAVE);
     scenarioDialog.setDirectory(directoryForRestrictions);
     scenarioDialog.setBackground(Color.white);
     scenarioDialog.show();
     String exclusionFile = scenarioDialog.getFile();
     String exclusionDirectory = scenarioDialog.getDirectory();
     if (exclusionFile != null) {
        try{writeExclusionVector(new File(exclusionDirectory,exclusionFile));}
         catch (IOException ioe){
            System.out.println("Error in saving restrictions");
         }
     }
  } 

  /** brings up Dialog for loading pre-existing Scenarios */
  
  public void showScenarioLoadDialog(){  
    // DRB: Just hard-code the filename as "scenario/specs" for now.
//      scenarioDialog = 
//          new FileDialog(parentFrame, "load scenarios", FileDialog.LOAD);
//      scenarioDialog.setDirectory(VerbLearn.DATA_BASE_DIR + "scenario");
//      scenarioDialog.setBackground(Color.white);
//      scenarioDialog.show();
//      String scenarioFile = scenarioDialog.getFile();
//      String scenarioDirectory = scenarioDialog.getDirectory();
 //   String scenarioFile = "specs";
 //   String scenarioDirectory = VerbLearn.DATA_BASE_DIR + "scenario";
 //    if (scenarioFile != null) {
 //        loadScenarios(new File(scenarioDirectory, scenarioFile));
 //    }
     String scenarioFilename = "scenario/specs";
     loadScenarios(scenarioFilename);
  }

  /** brings up Dialog for loading pre-existing restriction sets */
  
  public void showExclusionLoadDialog(){  
     scenarioDialog = 
         new FileDialog(parentFrame, "load restrictions", FileDialog.LOAD);
     scenarioDialog.setDirectory(directoryForRestrictions);
     scenarioDialog.setFile(standardRestrictions);
     scenarioDialog.setBackground(Color.white);
     scenarioDialog.show();
     String exclusionFile = scenarioDialog.getFile();
     String exclusionDirectory = scenarioDialog.getDirectory();
     //System.out.println(scenarioDirectory);
     //System.out.println(scenarioFile);
     if (exclusionFile != null) {
         loadOtherExclusions(new File(exclusionDirectory, exclusionFile));
     }
  }

  /** reads in the given File of restrictions */

  public void loadOtherExclusions(File file){
     int token = -8; 
     StreamTokenizer tokenizer;
     try {
          tokenizer = new StreamTokenizer(new FileInputStream(file)); 
          tokenizer.wordChars(' ', ' ');
          tokenizer.wordChars('=', '=');
          while (true) {
              try {token = tokenizer.nextToken();}
              catch (IOException ioe) {
                  System.out.println("Error: misformated restrictions file");
              }
              if (token == StreamTokenizer.TT_EOF) break;
              String string = tokenizer.sval;
              //System.out.println(string);
	      if (string != null){ 
                  exclusionVector.addElement(new Fstruct(string));
                  exclusionList.add(string);
              }
          }
     } 
     catch (FileNotFoundException fnfe){
          System.out.println("Requested restriction file not found");
     }
  }

  /** should load indicated file of Scenarios, but currently only works
      for default scenario file, "specs". The ScenarioSet class should
      be modified to be able to load other files.  */
  
  public void loadScenarios(File file){                                        
    // ScenarioSet ss = VerbLearn.context().scenarios();
     System.out.println(""+file);
     ScenarioSet ss = new ScenarioSet(""+file);
     Enumeration sEnum = ss.names();
     while (sEnum.hasMoreElements()){
        String sname = (String)sEnum.nextElement();
        Scenario sc = ss.get(sname);
        addScenario(sc);
     }
     updateButtonStates();
     updateSelectoMeter();
  } 

  public void loadScenarios(String filename){
     ScenarioSet ss = VerbLearn.context().scenarios();
     // ScenarioSet ss = new ScenarioSet(filename);
     Enumeration sEnum = ss.names();
     while (sEnum.hasMoreElements()){
        String sname = (String)sEnum.nextElement();
        Scenario sc = ss.get(sname);
        addScenario(sc);
     }
     updateButtonStates();
     updateSelectoMeter();
     displayScenario(scenarioChoice.getSelectedItem());
  }
     
  /** brings up dialog for choosing Params; currently restricted to 
      files located in "param" directory */
  
  //  this isn't currently used: using a choicebox instead
  
  public void showParameterFileDialog(){
     paramDialog = 
       new FileDialog(parentFrame, "select parameter file", FileDialog.LOAD);
     paramDialog.setDirectory(VerbLearn.DATA_BASE_DIR + "param");
     paramDialog.setBackground(Color.white);
     paramDialog.show();
     String paramFile = paramDialog.getFile();
     VerbLearn.context().setParams(paramFile);
     params = VerbLearn.context().params();
     //System.out.println(paramFile);
     buildFeatureList();
     buildWeightTable();
  }
  
  // nc added, based closely on above.
  /** brings up dialog for saving Params from current Context;
    currently restricted to files located in "param" directory (??) */
  // note: we specifically use this for case of adding/removing features
  // automatically switch to new file... looks a bit ugly, but...
  
  public void showParameterSaveDialog(){
    paramDialog = 
       new FileDialog(parentFrame, "select parameter file", FileDialog.SAVE);
    paramDialog.setDirectory(VerbLearn.DATA_BASE_DIR + "param");
    paramDialog.setBackground(Color.white);
    paramDialog.show();
    String paramFile = paramDialog.getFile();
    String paramDirectory = paramDialog.getDirectory();
    String pathname = paramDirectory + paramFile;
    //System.out.println(paramDirectory);
    //System.out.println(paramFile);
    if (paramFile != null) {
      try {
	//	writeParamVector(new File(paramDirectory, paramFile)); 
	PrintWriter pw = new PrintWriter(new FileWriter(pathname));
	pw.print(VerbLearn.context().params().toString());
	pw.close();
	
	VerbLearn.context().params().setId(paramFile);
	paramChoice.select(paramFile);
	VerbLearn.context().rereadLists(); // lists updated, calls update
	return;
      }
      catch (IOException ioe){
	System.out.println("Error in saving parameter file");
      }
    }
  }
  

/** updates things to use a different set of Params. */ 
 
  public void changeParams() {
    // should be current context, unless changed by user
    String paramString = paramChoice.getSelectedItem();
    // change to check if it's the current one.
    // null check might be redundant -- what if current is null?
    if (paramString != null && 
	paramString != VerbLearn.context().params().id())
      {
	VerbLearn.context().setParams(paramString);
      }
    params = VerbLearn.context().params();
    buildFeatureList();
    buildWeightTable();
  }
  
  /** launches PopupMenu for specifying value of given feature */
  
  public void buildValuePopup(String feature){
     valuePopup = new PopupMenu(feature);
     Vector vv = params.featureVals(feature);
     for (int i=0; i<vv.size(); i++){
        MenuItem tempItem = new MenuItem((String)vv.elementAt(i));
        valuePopup.add(tempItem);
     }
     valuePopup.addSeparator();
     valuePopup.add(new MenuItem("set randomly"));
     valuePopup.addSeparator();
     valuePopup.add(new MenuItem("adjust weights"));
     add(valuePopup);
     // figure out where to put the popup 
     // System.out.println(""+featureList.getVisibleIndex());
     // getVisibleIndex always returns -1 ?
     int offset = featureList.getSelectedIndex()- featureList.getVisibleIndex();
     int perRow = featureList.getSize().height / featureList.getRows();
     valuePopup.show(featureList, 80, (offset-1)*perRow );
  }
     
  /** launches popupMenu warning of improperly set TextField comp */
  
  public void buildErrorPopup(String text, Component comp){
     PopupMenu errorPopup = new PopupMenu(text);
     add(errorPopup);
     errorPopup.show(comp, 10, 10);
  }


  /** writes list of Scenarios to given File in a format that can be read 
      by VerbLearn */

  public void writeScenarioVector(File filename) throws IOException{
     int count = scenarioVector.size();
     Enumeration sEnum = scenSet.names();
     PrintStream p = 
            new PrintStream( new BufferedOutputStream( 
                              new FileOutputStream(filename)));
     p.println("// Generated scenarios:\n");  
     while (sEnum.hasMoreElements()) {
         Scenario sc = scenSet.get((String)sEnum.nextElement());
         p.println(sc.id() + "\t" + sc.initialLink().toString4IncludingOff()
		   + "\t" + sc.finalLink().toString4IncludingOff());     
     }
     p.println("// " + count + 
                        " outputs, " + rejectCount + " rejects.");
     p.flush();
  } 
  
  /** writes list of restrictions to given File in a format that can be
      read by loadExclusions */

  public void writeExclusionVector(File filename) throws IOException{
     int count = exclusionVector.size();
     Enumeration sEnum = exclusionVector.elements();
     PrintStream p = 
            new PrintStream( new BufferedOutputStream( 
                              new FileOutputStream(filename)));
     while (sEnum.hasMoreElements()) {
         Fstruct struct = (Fstruct)sEnum.nextElement();
         p.println(fstructToString(struct));     
     }
     p.flush();
  } 
  

  /** returns a one-line String representation of given Fstruct */

  public String fstructToString(Fstruct struct){
     String string = "";
     Enumeration fEnum = struct.features();
     while (fEnum.hasMoreElements()){
        String feature = (String)fEnum.nextElement(); 
        string+=(feature+"="+struct.get(feature)+" ");
     } 
     return string;
  }

  /** fills in featureList with features pulled from params */

  public void buildFeatureList(){
     featureList.removeAll();
     params = VerbLearn.context().params();
     Enumeration fEnum = params.worldFeatureNames();
     while (fEnum.hasMoreElements()) {
       String f = (String)fEnum.nextElement();
       f+="=";
       featureList.addItem(f);
     }
     fEnum = params.motorFeatureNames();
     while (fEnum.hasMoreElements()) {
        String f = (String)fEnum.nextElement();
        f+="=";
        featureList.addItem(f);
     } 
  }
  
  /** assembles Hashtable with weights (relative frequencies)
      of each value for each feature */

  public void buildWeightTable(){
     weightTable = new Hashtable(15);
     Enumeration fEnum = params.worldFeatureNames();
     while (fEnum.hasMoreElements()) {
        String f = (String)fEnum.nextElement();
        Hashtable tempHash = new Hashtable(6);
        Vector vv = params.featureVals(f);
        for (int i=0; i<vv.size(); i++){
            tempHash.put((String)vv.elementAt(i), new Integer(1));
        }
        weightTable.put(f, tempHash);
	// System.out.println(tempHash);
     } 
     fEnum = params.motorFeatureNames();
     while (fEnum.hasMoreElements()) {
        String f = (String)fEnum.nextElement();
        Hashtable tempHash = new Hashtable(6);
        Vector vv = params.featureVals(f);
        for (int i=0; i<vv.size(); i++){
            tempHash.put((String)vv.elementAt(i), new Integer(1));
        }
        weightTable.put(f, tempHash);
        // System.out.println(tempHash);
     } 
     // System.out.println(weightTable);
  }

  
  /** assembles all the set values in featureList into an Fstruct;
      ignores the features which haven't been set */

  public Fstruct extractRequiredFstruct(){
     String fstructString = "";
     for (int i=0; i<featureList.getItemCount(); i++){
        String temp = featureList.getItem(i);
        if(!temp.trim().endsWith("=")){
            fstructString += (temp+" ");
        }
     }
     Fstruct struct = new Fstruct(fstructString);
     return struct;
  }
  
  /** generates the desired number of Scenarios (with desired name-stem
      and starting index). If a feature-value is mandated in featureList,
      this'll have it. Otherwise a value will be chosen at random, with
      proabilities proportional to the weights. Scenarios which violate a 
      restriction in exclusionList are rejected, and we try again.
      This is now done by a separate thread.  */


  public void generateScenarios(){
    
    makeScenarioPopup();
    new generatorThread(this).start();

  }
 
  /** picks value for given feature at random, but with probabilities
      proportional to the weights */

  public String pickRandomValue(String feature){
     Vector vv = params.featureVals(feature);
     //System.out.println(feature);
     int totalWeight = 0;
     int accretingWeight = 0;
     Hashtable hash = (Hashtable)weightTable.get(feature);
     for (int i=0; i<vv.size(); i++) {
          totalWeight += ((Integer)hash.get(vv.elementAt(i))).intValue();
     } 
     int rand = (int)(Math.random()*totalWeight) + 1;
     //System.out.println(totalWeight + " " + rand);
     for (int i=0; i<vv.size(); i++){
         accretingWeight += ((Integer)hash.get(vv.elementAt(i))).intValue();
         if (rand <= accretingWeight) {
	     //System.out.println(" return " + vv.elementAt(i));
              return (String)vv.elementAt(i);
         }
     } 
     
     //shouldn't happen
     System.out.println("No winner");
     return (String)vv.elementAt(0);
  }


  
  /** adds given Scenario to scenarioVector, scenarioList, 
      scenarioTextArea */

  public void addScenario(Scenario sc){
     // remove filler
     if (scenarioChoice.getItemCount() == 1 && 
                 scenarioChoice.getItem(0).equals("<name>")) {
         scenarioChoice.remove(0);
     }
     // check if name's already in use
     nameStem = nameBox.getText().trim();
     if (nameInUse(sc.id())){
        int i=1;
        while(nameInUse(nameStem+i)) i++;
        System.out.println("renamed " + sc.id() + " to " + nameStem + i); 
        sc.setId(nameStem+i);
     }
     scenSet.addScenario(sc);
     scenarioChoice.add(sc.id());
     scenarioChoice.select(sc.id());
  }
  
  /** builds a Scenario from index and Fstructs; puts it in 
      appropriate places */

  public void addScenario(Fstruct w, Fstruct l, int i){
     String id = nameStem + (startingIndex+i);
     Scenario newScen = new Scenario(id, w, l);
     addScenario(newScen);
   }
    
  /** pops-up Dialog for adding restrictions */

  public void makeExclusionPopup(){
     exclusionPopupDialog exclusionDialog = 
         new exclusionPopupDialog(parentFrame, "Add Restrictions");
     exclusionDialog.show();      
  }

  /** pops-up dialog for adjusting weights */

  public void makeWeightsPopup(String feature){
     //String feature = featureList.getSelectedItem();

     //if (feature==null) return;
     //feature = feature.substring(0, feature.indexOf("="));
     // System.out.println(feature);
     weightsPopupDialog weightsDialog = 
         new weightsPopupDialog(parentFrame, feature);
     weightsDialog.show();     
  }
  
  /** pops-up dialog for cancelling scenario-generation in progress */

  public void makeScenarioPopup(){
    scenarioGenDialog = new scenarioPopupDialog(parentFrame, "Generating Scenarios...");
    scenarioGenDialog.show();
  }  
      
  
  /** returns true iff given Fstructs don't violate any of the restrictions 
      in exclusionList */   

  public boolean makesSense(Fstruct w, Fstruct l){
     Enumeration rEnum = exclusionVector.elements();
     while (rEnum.hasMoreElements()){
       Fstruct restriction = (Fstruct)rEnum.nextElement();
       if (violatesRestriction(l,restriction)){
          return false;
       }
     }
     return true;    
  }
  
  /** returns true iff testStruct violates the restriction given by verboten */

  public boolean violatesRestriction(Fstruct testStruct, Fstruct verboten){
        return (isWeakSubFstruct(testStruct, verboten));
   
  }
  
  /** returns true iff, for every feture which is is set in small, if it's 
      also set in big then the values agree. This is symmetric, I think */

  public boolean isWeakSubFstruct (Fstruct small, Fstruct big){
      Enumeration sEnum  = small.features();
      while (sEnum.hasMoreElements()){
         String feat = (String)sEnum.nextElement();
         if (big.isSet(feat) && !big.get(feat).equals(small.get(feat))){
             return false;
         }         
      }
      return true;
  }
  
  /** returns true iff every feature which is set in small is also set in big, 
      and the values of such features agree.  This isn't symmetric */

  public boolean isStrongSubFstruct (Fstruct small, Fstruct big){
      Enumeration sEnum  = small.features();
      while (sEnum.hasMoreElements()){
         String feat = (String)sEnum.nextElement();
         if (!big.isSet(feat) ||  !big.get(feat).equals(small.get(feat))){
             return false;
         }         
      }
      return true;
  }
    
  /** grabs the indices of the selected restrictions in exclusionList,
      and gets rid of the corresponding restrictions */

  public void removeSelectedExclusions(){
      int positions[] = exclusionList.getSelectedIndexes();
      for (int i = (positions.length) -1; i>=0; i--){
         exclusionList.remove(positions[i]);
         exclusionVector.removeElementAt(positions[i]);
      }
  }

  /** renames Scenarios to eliminates gaps caused by pruning.
      Uses current nameStem for all of them, and starts with index 1 */

  public void compactNames(){
     ScenarioSet tempSet = new ScenarioSet("nonexistent");
     nameStem = nameBox.getText();
     int count = scenarioChoice.getItemCount();
     for (int i=0; i<count; i++){
         String newName = nameStem+(i+1);
         Scenario scen = scenSet.get(scenarioChoice.getItem(i));
         scen.setId(newName);
         tempSet.addScenario(scen);
     }
     scenarioChoice.removeAll();
     for (int i=0; i<count; i++){
         scenarioChoice.add(nameStem+(i+1));
     }
     scenSet = tempSet;
     displayScenario(currentScenarioIndex);
     startingIndex = scenarioChoice.getItemCount()+1;
     firstNumberBox.setText(""+startingIndex);
  }

  
  /** deletes, with probability given by pruneBox, each
      selected Scenario in scenarioList */
 
   // no longer used
  public void pruneSelectedScenarios(){
     int pruneProb = 0;
     try {
        pruneProb = Integer.parseInt(pruneBox.getText().trim());
     }
     catch (NumberFormatException nfe) {
       // System.out.println("Error: prune factor needs to be an integer");
        buildErrorPopup("prune factor should be integer", pruneBox);
        return;
     }
     if (pruneProb > 100 || pruneProb < 0 ) {
       //System.out.println("Error: prune factor should be between 0 and 100");
      buildErrorPopup("prune factor should be between 0 and 100", pruneBox);
      return;
     }
     
     //else prune...                     
  }


  //  keeps indicated percent of selected scenarios selected.
  // deselects the rest.
  public void selectFractionOfSelected(int percent){
      for (int i = 0; i < selectedVector.size(); i++){
          int rand = (int)(Math.random()*100) + 1;
          if (rand > percent) selectedVector.removeElementAt(i);
      }
      updateSelectoMeter();
      updateButtonStates();
  }

   public void deleteSelectedScenarios(){
      // decide which scenario to display afterwards
      int index = scenarioChoice.getSelectedIndex();
      for (int j = index; j<scenarioChoice.getItemCount(); j++){
          if (!selectedVector.contains(scenarioChoice.getItem(j))){
             index = j;
             break; 
          } 
          index = 0;
      }
      String showMe = scenarioChoice.getItem(index);
      for (int i=0; i<selectedVector.size(); i++){
         String name = (String) selectedVector.elementAt(i);
         scenSet.removeScenario(name);
         scenarioChoice.remove(name);
      }
      selectedVector.removeAllElements();
      displayScenario(showMe);
   }
  
  /** prepares PopupMenu for selected feature in featureList */  

  public void setSelectedValue(){
     int index = featureList.getSelectedIndex();
     if (index == -1) return;
     String item = featureList.getSelectedItem();
     item = item.substring(0, item.indexOf("="));
     selectedFeature = item;
     selectedIndex= index;
     buildValuePopup(item);
  }

  public void handleValueMenuItemClick(String feature, String value, int index){
     if (value.equals("set randomly")) {
         setFeatureValue(feature, "", index);
     } else if (value.equals("adjust weights")){
         setFeatureValue(feature, "", index);
         makeWeightsPopup(feature);
     } else {
         setFeatureValue(feature, value, index);
     }
  }
 
  /** sets value of feature at index i to be value in featureList */

  public void setFeatureValue(String feature, String value, int index){
     featureList.replaceItem(feature+"="+value, index);
  }
   
  /** NC: Pops up dialog box for removing features */
  public void showRemoveFeatureDialog() {
    removeFeatDialog = new removeFeatureDialog(parentFrame, " ");
    removeFeatDialog.show();
  }

  public void displayScenario(String name){
    scenarioChoice.select(name);
    int index = scenarioChoice.getSelectedIndex();
    currentScenarioIndex = index;
    Scenario scen = (Scenario)(scenSet.get(name));
    if (scen != null) fstructViewer.display(scen);
    updateButtonStates();
    updateSelectoMeter();
  }


  // by index in scenarioChoice
  public void displayScenario(int index){
    scenarioChoice.select(index);
    String name = scenarioChoice.getSelectedItem();
    displayScenario(name);   
  }

  public void updateButtonStates(){
      if (currentScenarioIndex < scenarioChoice.getItemCount() -1)
           forward.setEnabled(true);
      else forward.setEnabled(false);
      if (currentScenarioIndex > 0)  rewind.setEnabled(true);
      else rewind.setEnabled(false);
      fastForward.setEnabled(false);
      for (int i=currentScenarioIndex+1; i<scenarioChoice.getItemCount(); i++){
          if (selectedVector.contains(scenarioChoice.getItem(i))){
             fastForward.setEnabled(true); 
             break;
          }
      }
      fastRewind.setEnabled(false);
      for (int i = currentScenarioIndex - 1 ; i >=0 ; i--){
          if (selectedVector.contains(scenarioChoice.getItem(i))){
             fastRewind.setEnabled(true); 
             break;
          }
      }
  }


  public void handleFeatureListClick(){
     int index = featureList.getSelectedIndex();
     if (index == -1) return;
     String item = featureList.getSelectedItem();
     item = item.substring(0, item.indexOf("="));
     selectedFeature = item;
     selectedIndex= index;
     buildValuePopup(item);
  }

  public void selectAllScenarios(){
     selectedVector.removeAllElements();
     for (int i=0; i<scenarioChoice.getItemCount(); i++) {
       selectedVector.addElement(scenarioChoice.getItem(i));
     }
     updateSelectoMeter();
     updateButtonStates();
  }

  public void selectNoScenarios(){
     selectedVector.removeAllElements();
     updateSelectoMeter();
     updateButtonStates();

  }

  public void selectRestrictedScenarios(){
     for (int i=0; i<scenarioChoice.getItemCount(); i++){
       Scenario sc = scenSet.get(scenarioChoice.getItem(i));
       
       if (sc!=null && !makesSense(sc.initialLink(), sc.finalLink())){
            if (!selectedVector.contains(scenarioChoice.getItem(i))){
                  selectedVector.addElement(scenarioChoice.getItem(i));
            }
       }
    }
    updateSelectoMeter(); 
    updateButtonStates(); 
  }
  
  public void selectComplementOfScenarios(){
     for (int i=0; i<scenarioChoice.getItemCount(); i++) {
         if (selectedVector.contains(scenarioChoice.getItem(i))){
             selectedVector.removeElement(scenarioChoice.getItem(i));
         } else {
             selectedVector.addElement(scenarioChoice.getItem(i));
         }    
     }
     updateSelectoMeter();
     updateButtonStates();
  }
  
  public void selectCurrentScenario(){
     String name = scenarioChoice.getSelectedItem();
     if (!selectedVector.contains(name)){
       selectedVector.addElement(name);
     } 
     updateSelectoMeter();
     updateButtonStates();
  }

  public void deselectCurrentScenario(){
     String name = scenarioChoice.getSelectedItem();
     if (selectedVector.contains(name)){
       selectedVector.removeElement(name);
     } 
     updateSelectoMeter();
     updateButtonStates();
  }

  public void updateSelectoMeter(){
     selectoMeter.setNumberOfThings(scenarioChoice.getItemCount());
     //System.out.println("count= "+scenarioChoice.getItemCount());
     selectoMeter.setCurrentIndex(currentScenarioIndex);
     //System.out.println("currentIndex= "+currentScenarioIndex);

     int k = selectedVector.size();
     //System.out.println("selected size= " +k);

     int[] indices = new int[k];
     int j=0;
     for (int i = 0; i<scenarioChoice.getItemCount(); i++){
        if (selectedVector.contains(scenarioChoice.getItem(i))){
           if (j<k) indices[j] = i;
           else System.out.println("Duplicates in scenarioChoice?");
           j++;
        }
     }
     selectoMeter.setSelectedIndices(indices);
     selectoMeter.repaint();
  }

  // index in scenarioChoice of scenario being displayed
  int currentScenarioIndex = 0;

   public boolean nameInUse(String name){
     return scenSet.namesVector().contains(name);
   }

   public void addNewScenario(){
     int i = 1;
     while (nameInUse("new"+i)) i++;
     String name = "new"+i;
     Scenario newSc = new Scenario(name, new Fstruct(""), new Fstruct(""));
     scenSet.addScenario(name, newSc);
     scenarioChoice.insert(name, currentScenarioIndex+1);
     currentScenarioIndex++;
     displayScenario(currentScenarioIndex);
  }

  public void deleteScenario(){
      String name = scenarioChoice.getSelectedItem();
      if (!nameInUse(name)) return;
      scenSet.removeScenario(name);
      scenarioChoice.remove(name);
      if (selectedVector.contains(name)) selectedVector.removeElement(name);
      if (scenarioChoice.getItemCount() == 0) scenarioChoice.add("<name>");
      // show next scenario, or previous if on last.
      if (currentScenarioIndex >= scenarioChoice.getItemCount()) {
          currentScenarioIndex = scenarioChoice.getItemCount() - 1;
      }
      if (currentScenarioIndex < 0) currentScenarioIndex = 0;
      displayScenario(currentScenarioIndex);
  }

  public void copyScenario(){
      int i = 1;
      while (nameInUse("copy"+i)) i++;
      String name = "copy"+i;
      Scenario currentSc = scenSet.get(scenarioChoice.getSelectedItem());
      Scenario newSc = 
         new Scenario(name, new Fstruct(currentSc.initialLink()),
                            new Fstruct(currentSc.finalLink()));
      scenSet.addScenario(name, newSc);
      scenarioChoice.insert(name, currentScenarioIndex+1);
      currentScenarioIndex++;
      displayScenario(currentScenarioIndex);
  }

 //merges scenarios with those in Context. Immediately
 //reflected in LabelPanel
  public void mergeIntoContext(){
     //System.out.println("g....");
     VerbLearn.context().scenarios().merge(scenSet);
     VerbLearn.context().updateScenarioSet();
  }
  


  ////////////////////////////////////////////////////////////

  /** event handling */

  public boolean handleEvent(Event e){
    if (e.id == Event.LIST_SELECT || e.id == Event.LIST_DESELECT){
        if (e.target == featureList) {
           handleFeatureListClick();
        }
    }
    return super.handleEvent(e);
  }

  public boolean action(Event e, Object o){
    if (e.target == saveButton){
        showScenarioSaveDialog(); 
        return true;
    }
    else if (e.target == genButton){
        generateScenarios(); 
        return true;
    }
    else if (e.target == loadButton){
        showScenarioLoadDialog();
        return true;
    }
  //    else if (e.target == addFeatureButton){ // nc added
  //      showAddFeatureDialog();
  //      return true;
  //    }
    else if (e.target == paramChoice){
        changeParams();
        return true;
    }
    else if (e.target == addExclusionButton){
        makeExclusionPopup();
        return true;
    }
    else if (e.target == removeExclusionButton){
        removeSelectedExclusions();
        return true;
    }
    else if (e.target == loadOtherExclusionsButton){
         showExclusionLoadDialog();
         return true;
    }
    else if (e.target == saveExclusionsButton){
         showExclusionSaveDialog();
         return true;
    }
    else if (e.target == changeFeatureButton){ // nc added
      showRemoveFeatureDialog();
      return true;
    }
    else if (e.target == valuePopup){
        return true;
    }
    else if (e.target instanceof MenuItem){
        if  (e.target == selAll) {
             selectAllScenarios(); 
             return true;
        } else if (e.target == selNone){
             selectNoScenarios(); 
             return true;
        } else if (e.target == selRestricted){
             selectRestrictedScenarios();
             return true;
        } else if (e.target == selComplement){ 
             selectComplementOfScenarios();
             return true;
        } else if (e.target == selFraction) {
            // System.out.println("not implemented");
             return true;
        } else if (e.target == selFrac25) {
             selectFractionOfSelected(25);
             return true;
        } else if (e.target == selFrac50) {
             selectFractionOfSelected(50);
             return true;
        } else if (e.target == selFrac75) {
             selectFractionOfSelected(75);
             return true;
        } else if (e.target == selCurrent) {
             selectCurrentScenario();
             return true;
        } else if (e.target == deselCurrent) {
             deselectCurrentScenario();
             return true;
        } else if (e.target == compactNames) {
             compactNames();
             return true;
        }else if (e.target == merge) {
             mergeIntoContext();
             return true;
        } else if (e.target == deleteSelected) {
             deleteSelectedScenarios();
             return true;
        } 

        else if (((MenuItem)e.target).getParent() == valuePopup){
             handleValueMenuItemClick(selectedFeature, o.toString(), selectedIndex);
             return true;
        } else return false;
    }
    else if (e.target == scenarioChoice){
        currentScenarioIndex = scenarioChoice.getSelectedIndex();
        displayScenario(scenarioChoice.getSelectedItem());
        return true;
    }
    else if (e.target == forward){
        currentScenarioIndex++;
        String name = scenarioChoice.getItem(currentScenarioIndex);
        displayScenario(name);
        return true;
    }
    else if (e.target == fastForward){
        for (int j=currentScenarioIndex+1; j<scenarioChoice.getItemCount(); j++){
         if (selectedVector.contains(scenarioChoice.getItem(j))){
             currentScenarioIndex = j;
             displayScenario(scenarioChoice.getItem(j));   
             break;     
         }
       }
       return true;
    }
    else if (e.target == rewind){
        currentScenarioIndex--;
        String name = scenarioChoice.getItem(currentScenarioIndex);
        displayScenario(name);
        return true;
    }
    else if (e.target == fastRewind){
       for (int j=currentScenarioIndex-1; j>=0; j--){
         if (selectedVector.contains(scenarioChoice.getItem(j))){
             currentScenarioIndex = j;
             displayScenario(scenarioChoice.getItem(j));   
             break;     
         }
       }
       return true;
    }
    else if (e.target == deleteScenario){
        deleteScenario();
        return true;
    } 
    else if (e.target == addNewScenario){
        addNewScenario();
        return true;
    } 
    else if (e.target == copyScenario){
       copyScenario();
       return true;
    }
    else if (e.target == scenarioSelect){
        buildSelectionPopup();
        selectionPopup.show(scenarioSelect, 20, 0);
        return true;
    } 
    else return false;
  }

  public boolean mouseDown(Event e, int x, int y){
     if (e.target == selectoMeter){
       int index = selectoMeter.clickedIndex;
       if (index!=currentScenarioIndex)displayScenario(index); 
       return true;
     }
     else return false;
  }
  
  public boolean mouseDrag(Event e, int x, int y){
     if (e.target == selectoMeter){
       int index = selectoMeter.clickedIndex;
       if (index!=currentScenarioIndex)displayScenario(index); 
       return true;
     }
     else return false;
  }


}

