package NTLSystem.verblearn;

import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import NTLSystem.support.*;


///////////////////////////////////////////////////////////////////////
// Recognition testing Panel

class RecognizePanel extends Panel {

  private static final String INITIAL_TEXT = "Recognition test log window\n\n";

  private Label title;
  private LangLabel langLabel;
  private Panel west, east;
  private Label cgLabel;
  private CheckboxGroup cg;
  private Checkbox all, selected;
  private Button doIt;
  private Label slLabel;
  private ScenarioList scenarioList;
  private ScrollingTextArea log;
  private Button clear, save;

  RecognizePanel() {
    setLayout(new GridBagLayout());
    title = new Label("Recognition Testing Window");
    title.setFont(new Font("Helvetica", Font.BOLD, 18));
    Constrain.add(this, title, 0, 0, 2, 1, 10, 10, 20, 10);
    langLabel = new LangLabel();
    Constrain.add(this, langLabel, 0, 1, 2, 1, 10, 10, 10, 10);
    west = new Panel();
    west.setLayout(new GridBagLayout());
    cgLabel = new Label("Recognize:");
    Constrain.add(west, cgLabel, 0, 0, 1, 1, 10, 10, 10, 10);
    cg = new CheckboxGroup();
    all = new Checkbox("All Recognition Test Examples", cg, true);
    Constrain.add(west, all, 0, 2, 1, 1, GridBagConstraints.NONE,
		  GridBagConstraints.WEST, 3, 10, 3, 10);
    selected = new Checkbox("Selected (w/ more detail)", cg, false);
    Constrain.add(west, selected, 0, 3, 1, 1, GridBagConstraints.NONE,
		  GridBagConstraints.WEST, 3, 10, 10, 10);
    doIt = new Button("Do It !");
    doIt.setFont(new Font("Helvetica", Font.PLAIN, 18));
    Constrain.add(west, doIt, 0, 4, 1, 1, 20, 10, 10, 10);
    Constrain.add(this, west, 0, 2, 1, 1, GridBagConstraints.BOTH,
		  GridBagConstraints.CENTER, 10, 10, 10, 10);
    east = new Panel();
    east.setLayout(new GridBagLayout());
    slLabel = new Label("Scenario List:");
    Constrain.add(east, slLabel, 0, 0, 1, 1, 10, 10, 10, 10);
    scenarioList = new ScenarioList(false, false, true);
    Constrain.add(east, scenarioList, 0, 1, 1, 1, 10, 10, 10, 10);
    Constrain.add(this, east, 1, 2, 1, 1, GridBagConstraints.BOTH,
		  GridBagConstraints.CENTER, 10, 10, 10, 10);
    log = new ScrollingTextArea(INITIAL_TEXT, 24, 80);
    Constrain.add(this, log, 0, 3, 2, 1, 10, 10, 10, 10);
    clear = new Button("Clear Log");
    Constrain.add(this, clear, 0, 4, 1, 1, GridBagConstraints.NONE,
		  GridBagConstraints.WEST, 5, 10, 10, 10);
    save = new Button("Save Log...");
    Constrain.add(this, save, 1, 4, 1, 1, GridBagConstraints.NONE,
		  GridBagConstraints.EAST, 5, 10, 10, 10);
  }

  public boolean action(Event e, Object junk) {
    if (e.target == selected) {
      scenarioList.setEnabled(true);
      return true;
    }
    if (e.target == all) {
      scenarioList.setEnabled(false);
      int[] selected = scenarioList.getSelectedIndexes();
      for (int i = 0; i < selected.length; i++) {
	scenarioList.deselect(selected[i]);
      }
      return true;
    }
    if (e.target == doIt) {
      recognize();
      return true;
    }
    if (e.target == clear) {
      log.setText(INITIAL_TEXT);
      return true;
    } 
    if (e.target == save) {
      if (VerbLearn.isApplication()) {
	saveLog();
      }
      else {
	log.append("Sorry, applet version can't save files.\n");
      }
      return true;
    }
    return super.action(e, junk);
  }

  private void recognize() {
    if (VerbLearn.context().params() == null) {
      log.append("You must choose a parameter file before testing " +
		 "recognition.\n");
      return;
    }
    if (cg.getSelectedCheckbox() == all && 
	VerbLearn.context().dataset() == null) {
      log.append("You haven't chosen a dataset!\n");
      return;
    }
    if (VerbLearn.context().lexicon() == null) {
      log.append("You must load or train a lexicon before testing " +
		 "recognition.\n");
      return;
    }
    String[] scArray = null;
    if (cg.getSelectedCheckbox() == selected) {
      scArray = scenarioList.getSelectedItems();
      if (scArray == null || scArray.length == 0) {
	log.append("You haven't chosen any scenarios to test!\n");
	return;
      }
    }
    log.append("Beginning recognition test...\n");
    int numCorrect = 0;
    int numTested = 0;
    Enumeration scEnum;
    if (cg.getSelectedCheckbox() == selected) {
      Vector v = new Vector();
      for (int i = 0; i < scArray.length; i++) {
	v.addElement(scArray[i]);
      }
      scEnum = v.elements();
    }
    else {
      scEnum = VerbLearn.context().dataset().recognizeEnum();
    }
    while (scEnum.hasMoreElements()) {
      String scName = (String)scEnum.nextElement();
      Scenario sc = VerbLearn.context().scenarios().get(scName);
      if (VerbLearn.context().scenarios().containsKey(scName)) {
	if (sc.isLabelled()) {
	  String actual = VerbLearn.context().lexicon().label(sc.finalLink())
	    .toString();
	  log.append("Scenario " + sc.id() + ": desired=" + sc.label() + 
		     ", output=" + actual);
	  numTested++;
	  if (actual.equals(sc.label().toString())) {
	    numCorrect++;
	  }
	  else {
	    log.append("    *ERROR*");
	  }
	  log.append("\n");
	  // Print out full word probability vector only if in "selected" mode:
	  if (cg.getSelectedCheckbox() == selected) {
	    log.append(VerbLearn.context().lexicon()
		       .allSensePosteriors(sc.finalLink()));
	  }
	}
	else {
	  log.append("WARNING: scenario " + sc.id() + " is unlabelled.\n");
	}
      }
      else {
	log.append("WARNING: non-existing scenario" + scName +
		   " not in scenario set.\n");
      }
    }
    log.append("Correctly labelled " + numCorrect + " of " +
	       numTested + " test scenarios");
    if (numTested > 0) {
      log.append(" (" +
		 ((int)((double)numCorrect / (double)numTested * 100)) + 
		 "%).\n");
    }
    log.append("Recognition test done.\n\n");
  }

  private void saveLog() {
    if (VerbLearn.context().langslot() != null) {
      String dirname = VerbLearn.DATA_BASE_DIR + "lang/" +
	VerbLearn.context().langslot() + "/output/";
      File f = new File(dirname);
      if (!f.exists()) {
	f.mkdirs();
      }
      FileDialog fd = new FileDialog(VerbLearn.frame(), 
				     "Save Recognition Test Log As...", 
				     FileDialog.SAVE);
      fd.setDirectory(dirname);
      fd.show();
      String fname = fd.getFile();
      if (fname != null) {
	String pathname = fd.getDirectory() + fname;
	try {
	  PrintWriter pw = new PrintWriter(new FileWriter(pathname));
	  pw.print(log.getText());
	  pw.close();
	  log.append("This log saved in file " + pathname + "\n");
	}
	catch (IOException e) {
	  log.append("Could not save this log in " + pathname + ": " + 
		     e.getMessage() + "\n");
	}
      }
    }
  }
}
